#
# Copyright (c) 2007 Red Hat
#
# Written by Mauricio Teixeira <mteixeira@webset.net>
#
# This file is part of Smart Package Manager.
#
# Smart Package Manager is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as published
# by the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
#
# Smart Package Manager is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Smart Package Manager; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
import posixpath
import md5
import os
import ConfigParser
import re

# be compatible with 2.3
import sys
if sys.version_info < (2, 4):
    from sets import Set as set

from smart.channel import *
from smart import *
 
YUM_REPOS_DIR = "/etc/yum.repos.d/"

import rpmUtils.arch
BASEARCH = rpmUtils.arch.getBaseArch()
RELEASEVER = None

def _getreleasever():

    import rpm
    import rpmUtils.transaction

    rpmroot = sysconf.get("rpm-root", "/")
    ts = rpmUtils.transaction.initReadOnlyTransaction(root=rpmroot)
    ts.pushVSFlags(~(rpm._RPMVSF_NOSIGNATURES|rpm._RPMVSF_NODIGESTS))
    # HACK: we're hard-coding the most used distros
    idx = ts.dbMatch('provides', 'fedora-relase')
    if idx.count() == 0:
        idx = ts.dbMatch('provides', 'redhat-release')
    if idx.count() != 0:
        hdr = idx.next()
        releasever = hdr['version']
        del hdr
    del idx
    del ts
    return str(releasever)

def _loadRepoFile(filename):

    file = open(filename)
 
    # The computed aliases we have seen in the given file.
    seen = set()

    repofile = ConfigParser.ConfigParser()
    repofile.read(filename)

    for repo in repofile.sections():
        alias = "yumsync-%s" % repo
        name = re.sub("\$basearch", "%s" % BASEARCH,
               repofile.get(repo, 'name'))
        name = re.sub("\$releasever", "%s" % RELEASEVER,
               name)
        baseurl = re.sub("\$basearch", "%s" % BASEARCH,
                  repofile.get(repo, 'baseurl'))
        baseurl = re.sub("\$releasever", "%s" % RELEASEVER,
                  baseurl)
        data = {"type": "rpm-md",
                "name": name,
                "baseurl": baseurl,
                "enabled": repofile.get(repo, 'enabled')}
        seen.add(alias)
 
        try:
            createChannel(alias, data)
        except Error, e:
            iface.error(_("While using %s: %s") % (filename, e))
        else:
            # Store it persistently.
            sysconf.set(("channels", alias), data)

    return seen


def syncYumRepos(reposdir, force=None):

    seen = set()

    if os.path.isdir(reposdir):
        for entry in os.listdir(reposdir):
            if entry.endswith(".repo"):
                filepath = os.path.join(reposdir, entry)
                if os.path.isfile(filepath):
                    seen.update(_loadRepoFile(filepath))

    # Delete the entries which were not seen in current files.
    channels = sysconf.get("channels")
    for alias in sysconf.keys("channels"):
        if alias.startswith("yumsync-") and alias not in seen:
            sysconf.remove(("channels", alias))


if not sysconf.getReadOnly():
    if sysconf.get("sync-yum-repos",False):
       RELEASEVER = _getreleasever()
       syncYumRepos(sysconf.get("yum-repos-dir", YUM_REPOS_DIR))
